@@ -1,15 +1,17 @@
-from typing import TYPE_CHECKING, Callable, List, Optional, Tuple, Union
-from urllib.parse import unquote
+import typing
 
-import httpcore
 import sniffio
 
-if TYPE_CHECKING:  # pragma: no cover
+from .._models import Request, Response
+from .._types import AsyncByteStream
+from .base import AsyncBaseTransport
+
+if typing.TYPE_CHECKING:  # pragma: no cover
     import asyncio
 
     import trio
 
-    Event = Union[asyncio.Event, trio.Event]
+    Event = typing.Union[asyncio.Event, trio.Event]
 
 
 def create_event() -> "Event":
@@ -23,7 +25,15 @@ def create_event() -> "Event":
         return asyncio.Event()
 
 
-class ASGITransport(httpcore.AsyncHTTPTransport):
+class ASGIResponseStream(AsyncByteStream):
+    def __init__(self, body: typing.List[bytes]) -> None:
+        self._body = body
+
+    async def __aiter__(self) -> typing.AsyncIterator[bytes]:
+        yield b"".join(self._body)
+
+
+class ASGITransport(AsyncBaseTransport):
     """
     A custom AsyncTransport that handles sending requests directly to an ASGI app.
     The simplest way to use this functionality is to use the `app` argument.
@@ -58,46 +68,40 @@ class ASGITransport(httpcore.AsyncHTTPTransport):
 
     def __init__(
         self,
-        app: Callable,
+        app: typing.Callable,
         raise_app_exceptions: bool = True,
         root_path: str = "",
-        client: Tuple[str, int] = ("127.0.0.1", 123),
+        client: typing.Tuple[str, int] = ("127.0.0.1", 123),
     ) -> None:
         self.app = app
         self.raise_app_exceptions = raise_app_exceptions
         self.root_path = root_path
         self.client = client
 
-    async def arequest(
+    async def handle_async_request(
         self,
-        method: bytes,
-        url: Tuple[bytes, bytes, Optional[int], bytes],
-        headers: List[Tuple[bytes, bytes]] = None,
-        stream: httpcore.AsyncByteStream = None,
-        ext: dict = None,
-    ) -> Tuple[int, List[Tuple[bytes, bytes]], httpcore.AsyncByteStream, dict]:
-        headers = [] if headers is None else headers
-        stream = httpcore.PlainByteStream(content=b"") if stream is None else stream
+        request: Request,
+    ) -> Response:
+        assert isinstance(request.stream, AsyncByteStream)
 
         # ASGI scope.
-        scheme, host, port, full_path = url
-        path, _, query = full_path.partition(b"?")
         scope = {
             "type": "http",
             "asgi": {"version": "3.0"},
             "http_version": "1.1",
-            "method": method.decode(),
-            "headers": headers,
-            "scheme": scheme.decode("ascii"),
-            "path": unquote(path.decode("ascii")),
-            "query_string": query,
-            "server": (host.decode("ascii"), port),
+            "method": request.method,
+            "headers": [(k.lower(), v) for (k, v) in request.headers.raw],
+            "scheme": request.url.scheme,
+            "path": request.url.path,
+            "raw_path": request.url.raw_path,
+            "query_string": request.url.query,
+            "server": (request.url.host, request.url.port),
             "client": self.client,
             "root_path": self.root_path,
         }
 
         # Request.
-        request_body_chunks = stream.__aiter__()
+        request_body_chunks = request.stream.__aiter__()
         request_complete = False
 
         # Response.
@@ -138,7 +142,7 @@ class ASGITransport(httpcore.AsyncHTTPTransport):
                 body = message.get("body", b"")
                 more_body = message.get("more_body", False)
 
-                if body and method != b"HEAD":
+                if body and request.method != "HEAD":
                     body_parts.append(body)
 
                 if not more_body:
@@ -154,7 +158,6 @@ class ASGITransport(httpcore.AsyncHTTPTransport):
         assert status_code is not None
         assert response_headers is not None
 
-        stream = httpcore.PlainByteStream(content=b"".join(body_parts))
-        ext = {}
+        stream = ASGIResponseStream(body_parts)
 
-        return (status_code, response_headers, stream, ext)
+        return Response(status_code, headers=response_headers, stream=stream)
